
const express = require('express');
const logger = require('cyber-express-logger');
const axios = require('axios');
const ColorThief = require('colorthief');
const utils = require('web-resources');
const config = require('./config.json');

const srv = express();
srv.set('view engine', 'ejs');
srv.use(logger({ getIP: req => req.headers['cf-connecting-ip'] }));
srv.use('/assets', express.static('assets'));

srv.use((req, res, next) => {
    res.data = {
        siteName: `Discord User Info`,
        title: ``,
        desc: ``,
        image: ``,
        user: null,
        id: null
    };
    res.renderPage = () => res.render('main', res.data);
    next();
});

srv.get('/', (req, res) => {
    res.renderPage();
});

const users = {};
const getUserById = async id => {
    // Return saved user info if we have it and it's not expired
    if (users[id] && users[id].expires > Date.now()) return users[id];
    // Request user info from Discord
    const res = await axios.get(`https://discord.com/api/v9/users/${id}`, {
        headers: {
            Authorization: `Bot ${config.discord.token}`,
            'User-Agent': `DiscordBot (https://discordusers.simplecyber.org, 1.0.0)`
        },
        validateStatus: status => true
    });
    // Build user object
    users[id] = { ...res.data, expires: (Date.now()+(1000*60*15)) };
    delete users[id].display_name;
    delete users[id].flags;
    // If the user has a username (is valid)
    if (users[id].username) {
        // Set the user's avatar_url
        if (users[id].avatar) {
            users[id].avatar_url = `https://cdn.discordapp.com/avatars/${users[id].id}/${users[id].avatar}.${(users[id].avatar.match(/^a_/)) ? 'gif' : 'png'}?size=512`;
        } else {
            users[id].avatar_url = `https://cdn.discordapp.com/embed/avatars/${users[id].discriminator % 5}.png`;
        }
        // Get the user's accent color from their avatar if they don't have one set
        if (users[id].accent_color) {
            users[id].accent_color = `#${users[id].accent_color.toString(16).padStart(6, '0')}`;
        } else {
            const color = await ColorThief.getColor(users[id].avatar_url);
            users[id].accent_color = `#${color[0].toString(16)}${color[1].toString(16)}${color[2].toString(16)}`;
        }
        // Set the user's banner color to their accent color if it isn't set
        if (!users[id].banner_color) users[id].banner_color = users[id].accent_color;
        // If the user has a banner, get the banner url
        if (users[id].banner) {
            users[id].banner_url = `https://cdn.discordapp.com/banners/${users[id].id}/${users[id].banner}.${(users[id].banner.match(/^a_/)) ? 'gif' : 'png'}?size=1024`;
        } else {
            users[id].banner_url = null;
        }
        // Save if the user is a bot
        users[id].is_bot = Boolean(users[id].bot);
        delete users[id].bot;
        // Calculate the user's flags
        // https://discord.com/developers/docs/resources/user#user-object-user-flags
        const flags = [
            { id: 'STAFF', friendly: 'Discord Employee', bit: 0 },
            { id: 'PARTNER', friendly: 'Discord Partner', bit: 1 },
            { id: 'HYPESQUAD', friendly: 'HypeSquad Events Member', bit: 2 },
            { id: 'BUG_HUNTER_LEVEL_1', friendly: 'Bug Hunter Level 1', bit: 3 },
            { id: 'HYPESQUAD_ONLINE_HOUSE_1', friendly: 'HypeSquad Bravery', bit: 6 },
            { id: 'HYPESQUAD_ONLINE_HOUSE_2', friendly: 'HypeSquad Brilliance', bit: 7 },
            { id: 'HYPESQUAD_ONLINE_HOUSE_3', friendly: 'HypeSquad Balance', bit: 8 },
            { id: 'PREMIUM_EARLY_SUPPORTER', friendly: 'Early Supporter', bit: 9 },
            { id: 'TEAM_PSEUDO_USER', friendly: '', bit: 10 },
            { id: 'BUG_HUNTER_LEVEL_2', friendly: 'Bug Hunter Level 2', bit: 14 },
            { id: 'VERIFIED_BOT', friendly: 'Verified Bot', bit: 16 },
            { id: 'VERIFIED_DEVELOPER', friendly: 'Verified Bot Developer', bit: 17 },
            { id: 'CERTIFIED_MODERATOR', friendly: 'Moderator Programs Alumni', bit: 18 },
            { id: 'BOT_HTTP_INTERACTIONS', friendly: '', bit: 19 },
            { id: 'ACTIVE_DEVELOPER', friendly: 'Active Developer', bit: 22 }
        ];
        users[id].public_flags_ids = [];
        users[id].public_flags_friendly = [];
        for (const flag of flags) {
            if (users[id].public_flags & (1 << flag.bit)) {
                users[id].public_flags_ids.push(flag.id);
                if (flag.friendly)
                    users[id].public_flags_friendly.push(flag.friendly);
            }
        }
        // Save if this user has a new username
        users[id].is_new_username = users[id].discriminator == '0';
        // Set the user's "tag"
        users[id].tag = users[id].is_new_username ? users[id].username : `${users[id].username}#${users[id].discriminator}`;
        // Calculate the user's creation timestamp
        users[id].time_created = Number(BigInt(users[id].id) >> 22n) + 1420070400000;
    }
    console.log(`Got user data for ${id}:`, users[id]);
    return users[id];
};

srv.get('/u/:id', async(req, res) => {
    res.data.id = req.params.id;
    res.data.user = await getUserById(req.params.id);
    if (res.data.user) {
        res.data.title = res.data.user.global_name || res.data.user.username;
        res.data.image = res.data.user.avatar_url;
        res.data.desc = `${res.data.user.tag} has been on Discord for ${utils.getRelativeDate(res.data.user.time_created).replace(' ago', '')}. Click to learn more about their account!`;
    }
    res.renderPage();
});

srv.get('/u/:id/avatar', async(req, res) => {
    const user = await getUserById(req.params.id);
    if (!user) return res.status(404).end();
    res.redirect(user.avatar_url);
});
srv.get('/u/:id/banner', async(req, res) => {
    const user = await getUserById(req.params.id);
    if (!user) return res.status(404).end();
    res.redirect(user.banner_url);
});

srv.get('/u/:id/json', async(req, res) => {
    res.setHeader('access-control-allow-origin', '*');
    const user = await getUserById(req.params.id);
    if (!user) return res.status(404).end();
    res.json(user);
});

srv.use((req, res) => {
    res.status(404).end();
});

srv.listen(config.port, console.log(`Listening on port ${config.port}`));